"""

Forces in the energy system:

* Demand. Seems the driving force. But it must adapt to feasibility (of final configurations and
  the actionability of transitions)
* Economic. Investment and operation costs of final configuration and the transition to it.
* Technical. The scenario must meet all technical constraints. For instance, RE penetration is usually limited to
  a percentage of total demand.
* Social. jobs, decrease of energy-poverty, energy self-sufficiency

Generate a Calliope model.

Which information is available?
* impact of technologies. A balance of impacts may be performed


"""
